package com.example.nto.service.impl;

import com.example.nto.dto.EmployeeDTO;
import com.example.nto.dto.EnterDTO;
import com.example.nto.entity.Code;
import com.example.nto.entity.Employee;
import com.example.nto.entity.Enter;
import com.example.nto.entity.EnterType;
import com.example.nto.exception.EmployeeNotFoundException;
import com.example.nto.exception.IncorrectCodeException;
import com.example.nto.exception.IncorrectTypeException;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.repository.EnterRepository;
import com.example.nto.repository.EnterTypeRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.util.EmployeeMapper;
import com.example.nto.util.EnterMapper;
import liquibase.pro.packaged.E;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import static org.springframework.util.Assert.hasText;

@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {

    private final EmployeeRepository employeeRepository;
    private final CodeRepository codeRepository;
    private final EnterTypeRepository enterTypeRepository;
    private final EnterRepository enterRepository;

    @Override
    public EmployeeDTO getEmployeeByUsername(String username) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);

        if(optionalEmployee.isEmpty())
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");

        return EmployeeMapper.convertToDTO(optionalEmployee.get());
    }

    @Override
    public EmployeeDTO getInfo(String username) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);
        if(optionalEmployee.isEmpty()) {
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");
        }

        return EmployeeMapper.convertToDTO(optionalEmployee.get());
    }

    @Override
    public EnterDTO addEnter(String username, EnterDTO enterDTO) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);
        if(optionalEmployee.isEmpty())
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");
        Optional<Code> optionalCode = codeRepository.findByValue(enterDTO.getValue());
        if(optionalCode.isEmpty())
            throw new IncorrectCodeException("Incorrect code");

        Optional<EnterType> optionalEnterType = enterTypeRepository.findByType(enterDTO.getType());
        if(optionalEnterType.isEmpty())
            throw new IncorrectTypeException("Incorrect type");

        Enter enter = new Enter();
        enter.setType(optionalEnterType.get());
        enter.setTime(enterDTO.getTime());
        enter.setValue(enterDTO.getValue());
        enter.setUsername(username);


        return EnterMapper.convertToDTO(enterRepository.save(enter));
    }

    @Override
    public List<EnterDTO> getAllEmployeeEnters() {
        return List.of();
    }


}
